
	MODULE poisson_routines

	implicit none

! Global Variables

	integer,  parameter		:: DP=kind(1.d0)
	real(DP), parameter		:: pi=3.141592653589793238462643383279502884197_dp

	integer,  parameter		:: ITMAX   = 100000	!stop if exceed this # of iterations
	integer,  parameter		:: NOUTPUT = 1000	!output after this # iterations

      	character (len=128)             :: filename_head = 'poisson.dat'


	CONTAINS

!==========================================================================

      	Function makerho(L,N)

!  initialize potential to a positive and negative guassian 
!  centered at (x=L/4,y=0) and (x=3/4*L,y=0), repsectively.

      	implicit none

	integer,	intent(in)	:: N
	real(DP),	intent(in)	:: L
	real(DP)	  		:: makerho(N,N)

! Local Variables

      	integer 			:: i,j
      	real(DP) 			:: a, x, y

! initialize width of gaussian

      	a = -16./L/L*log(0.1)

! create the double peaked gaussian
      
      	do j = 1, N
           y = (real(j)-.5)/N * L-L/2.
           do i = 1,N
              x = (real(i)-.5)/N*L
              makerho(i,j) = a/pi*(exp(-a*((x-L/4.)**2+y*y)) - &
              exp(-a*((x-3*l/4.)**2+y*y)))
           enddo
        enddo

      	end function makerho


!==========================================================================

	subroutine output( phi, filename_head, it )

	use string_utilities
	
	implicit none

	integer,		intent(in) 	:: it
	real(DP),		intent(in) 	:: phi(:,:)
	character(128),		intent(in) 	:: filename_head

!  Local Variables

	integer					:: i, j, N
	character(128)				:: filename

!  output file with name ' filename_head.*** ' where
!  *** is a 3 digit character string representation
!  of the integer i, with leading zeros, i.e. 1 -> 001, etc...

!  To plot with gnuplot type: gnuplot
!  At the gnuplot prompt: gnuplot>
!  type     load "gnuplot_poisson"
!  (the quotes are needed).
!  Make sure the file gnuplot_poisson points to the correct
!  output file to plot (.e.g. poisson.dat.4789)

	N = size(phi,1)
        filename = int_to_char_suffix( filename_head, it )
        open(1, file = filename)
           do i = 1, N
              do j = 1, N
                 write(1,*) i, j, real( phi(i, j) )
              enddo
	      write(1,*) '#' !for use with gnuplot
           enddo
        close(1)
        print*,'Final output is in file -> ',trim(filename)

!  also write out the output in binary format

	open(1,file=trim(filename)//'.bin',form='unformatted',access='direct',recl=4*N*N)
        write(1,rec=1) real( phi )
        close(1)
	   		
	end subroutine output

!==========================================================================

	END MODULE poisson_routines
